//
//  M A G N E T O
// ===============
// Version       : 0.7
// Start         : 14.09.2022
// nderung      : 14.04.2023
// IDE           : Microschip Studio 7.0.2594
// Controller    : ATMEGA1284 (war ATMEGA644. Upgrade wegen Speicherplatz. Pincompatibel)
// Quartz        : 14.745.600 Hz (hchstmgliche Frequenz fr fehlerfreie Datenbertragung
// Schnittstelle : UART mit 115.000 Baud.
// Code          : Codeteile, die aus dem Netz kopiert wurden, sind mit Quelle bzw. Autor versehen.
// Funktion      : Joystick zum Projekt MAGNETO. Hiermit kann der Sensor direkt an die Spulen 
//                 herangefahren und Messpunkte gespeichert werden. Aktiviert wird der Joystick
//                 ber die Visualisierung. Danach sendet der Joystick kontinuierlich die Zustnde
//                 der beiden Joysticks (0-100% Ausschlag). Das bestimmt die Geschwindigkeit des
//                 Roboterarms. Auch die vier Tasten "freie Fahrt", "Programmieren", "Automatik"
//                 und "Strung" wird zurckgemeldet.
//
// offen	     : 87 - Joystick Portal nicht implementiert.
//               : 88 - Schnittstellen Probleme bei Aktivierung aller Komponenten.
//               : 89 - Menstruktur fehlt, aktuell nur zu Testzwecken.
//               : 90 - Dreh-Encoder Men noch nicht implementiert.
//               : 91 - Tasten blinken Flag noch realisieren.
//				 : 92 - Erst wenn einer der Freigabe Taster, links oder rechts gedrckt wurde,
//				 :      werden die Joystick Werte im Protokoll versandt.
//
// ACHTUNG       : Bei den Einstellungen unter Toolchain "AVR GNU C Compiler / Symbols" muss F_CPU=14745600UL
//                 eingetragen werden, da es sonst bei der Verwendung von util/delay.h zu Konflikten kommt.
//

#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include "Main.h"

int main(void)
{
	Ports_INIT();					// Alle Ein- Ausgnge definieren.
	RS232_0_INIT();					// Schnittstelle fr RS485 initialisieren.
	SPI_MasterInit();				// SPI Schnittstelle initialisieren.
	LEDs_INIT();					// Alle LEDs initialisieren und ausschalten.
	Display_INIT();					// Das Display und SPI initialisieren.
	Joystick_INIT();				// Die Joysticks und den AD Wandler initialisieren. 
	Encoder_INIT();					// Die Encoder und Timer Interrupt initialisieren.
	Timer_LED_INIT();				// Fr den Blinktakt der LEDs.

	RS485_Adresse_festlegen();		// Fr diesen Busteilnehmer wurden keine DIP Switches verbaut.
									// Der Joystick erhlt die feste Busadresse 10.
	
	sei();							// Nach der Initialisierung der Hardware
									// alle Interrupts freigeben.

	Taste_freie_Fahrt = true;		// Taste 'freie Fahrt' beim Start aktivieren.
	LED_T_freie_Fahrt_EIN();		// Die LED dazu einschalten.

	// Joystick gestartet und bereit.
	// 3 x blinken der Status LED.
	// Ein Delay an der Stelle ist nicht kritisch,
	// wir sind noch nicht in der Hauptschleife.
	for (int a=0; a > 3; a++)
	{
		LED_Status_EIN();
		_delay_ms(250);
		LED_Status_AUS();
		_delay_ms(250);
	}

	char RS485_Rueckmeldung[255];				// Maximal 256 Zeichen die Rckmeldung an die Visu.
	int RS485_Rueckmeldung_Laenge;				// Stringlnge muss zum Schluss ermittelt werden, da diese Teil vom Protokoll ist.
	char RS485_Rueckmeldung_Laenge_String[5];	// Die Lnge des Strings wird in ASCII Zeichen fr das Protokoll bentigt.

	// Nach dem Programmstart auf Empfang gehen und auf Kommondos warten.
	RS485_auf_Empfang_schalten();

	//
	// Hauptprogramm als Endlosschleife
	//
	while(1)
	{
		Joystick_Sensor_abfragen();			// Alle 3 Achsen ber die AD Wandler abfragen
											// und auf 0-100% skalieren.
	
		//Joystick_Portal_abfragen();		// Noch nicht implementiert. Wird eine Kopie
											// vom Joystick_Sensor.
	
		Encoder_Schritte_abfragen();		// Dreh-Encoder abfragen (Interrupt-Betrieb)
											// und am Display anzeigen. (Vollschritte bis 256 Schritte
											// in 8 Stufen.
	
		//Encoder_Menu_abfragen();			// Noch nicht implementiert. Wird eine Kopie
											// vom Encoder_Schritte.
		
		Tasten_Code = Tasten_einlesen();
		if (Tasten_Code != 0xFF) Tasten_auswerten();
	
		// Nur zu Testzwecken, ob das Blinken funktioniert (500ms Takt).
		// Jede Taste bekommt noch ein Blinken-Flag!	
		if( LED_blinken == 1 )
		{
			LED_T_Stoerung_EIN();
			LED_Freigabe_EIN();
			LED_T_Freigabe_Links_EIN();
			LED_T_Freigabe_Rechts_EIN();
		}
		else
		{
			LED_T_Stoerung_AUS();
			LED_Freigabe_AUS();
			LED_T_Freigabe_Links_AUS();
			LED_T_Freigabe_Rechts_AUS();
		}

		//
		// Wenn eine Nachricht mit einem LF '\n' am Ende empfangen wurde,
		// dann auswerten, Rckantwort zusammenstellen und abschicken.
		//
		if (RS485_Empfang_komplett == 1)
		{
			Nachricht_Auswertung();
		
			if (Nachricht_vorhanden == true)	// Wenn ein korrekter Befehl mit Daten empfangen wurde,
			{									// dann Befehl ausfhren.
				Befehl_ausfuehren();
						
				if (Nachricht_OK == true)
				{
					strcpy(Nachricht_Rueckmeldung, Nachricht_Befehl);
					strcat(Nachricht_Rueckmeldung, "=");
					strcat(Nachricht_Rueckmeldung, Nachricht_Daten);
				}
			
				strcpy(RS485_Rueckmeldung, Nachricht_Startzeichen);
				strcat(RS485_Rueckmeldung, Nachricht_Sender);
				strcat(RS485_Rueckmeldung, ":");
				strcat(RS485_Rueckmeldung, Nachricht_Empfaenger);
				strcat(RS485_Rueckmeldung, ":");
				RS485_Rueckmeldung_Laenge = 8 + strlen(Nachricht_Rueckmeldung) + 6;
				strcat(RS485_Rueckmeldung, Nachricht_Rueckmeldung);
				strcat(RS485_Rueckmeldung, ":");
				sprintf(RS485_Rueckmeldung_Laenge_String, "%03i", RS485_Rueckmeldung_Laenge);
				strcat(RS485_Rueckmeldung, RS485_Rueckmeldung_Laenge_String);
				strcat(RS485_Rueckmeldung, Nachricht_Endzeichen);
				strcat(RS485_Rueckmeldung, "\n");
		
				uart0_puts(RS485_Rueckmeldung);
			
				
			} // Nachricht = vorhanden
			else
			{
				uart0_puts("FEHLER\n");			// Nur zu Testzwecken, wegen dem sporadischen UART Fehler.
			}
	
			
			RS485_Empfang_Buffer[0] = '\0';		// Den Empfangsbuffer wieder an den Anfang setzen.
			RS485_Empfang_komplett = 0;
			Nachricht_vorhanden = false;
	
			Nachricht_Befehl[0] = '\0';
			Nachricht_Daten[0] = '\0';
	
			RS485_auf_Empfang_schalten();		// Erst jetzt knnen wieder Daten empfangen werden.

	
		} //RS485_Empfang_komplett
	} //while()
	
	// Nirvana!
	return 0;
} // main()